

import markdown2 as md


md2_extras = [
    'break-on-newline',
    'code-friendly',
    'cuddled-lists',
    'fenced-code-blocks',
    'footnotes',
    'header-ids',
    'smarty-pants',
    'strike',
    'tables',
    'tg-spoiler',
    'tag-friendly',
    'task_list',
]


def marquedown(document: str, /, **kwargs) -> str:
    """
    Convert both Marquedown and Markdown into HTML.
    """

    if kwargs.get('citation', True):
        from .citation import citation
        document = citation(document)

    if kwargs.get('labellist', True):
        from .labellist import labellist
        document = labellist(document)

    if kwargs.get('qrgen') is not None:
        # Parse QR codes with provided QRGenerator
        from .qr import qr
        document = qr(document, kwargs.get('qrgen'))

    if kwargs.get('video', True):
        from .video import video
        document = video(document)
        
    html = md.markdown(document, extras=md2_extras)

    if kwargs.get('bblike', True):
        from .bblike import bblike
        html = bblike(html)

    return html


# Render all Markdown and Marquedown documents
# from a specified directory and all its subdirectories.
# 
# Provide a template for embedding the rendered content.

import re
import os
import yaml

from pathlib import Path
from collections import defaultdict
from typing import Generator

from .. import marquedown
from ..qr import QRGenerator


def extract_metadata(document: str) -> tuple[dict, str]:
    """
    Extract metadata from a YAML code block at the top of the document.
    
    Example:
        ```yml
        title: Page Title
        ```
    """

    m = re.search(r'^\`{3}ya?ml\n((?:.|\n)*?)\n\`{3}\s+', document)

    # Return without metadata if missing
    if not m:
        return {}, document

    # Remove metadata from document
    document = document[len(m.group(0)):]

    # Attempt to load metadata from YAML
    metadata = yaml.safe_load(m.group(1)) or {}

    if not isinstance(metadata, dict):
        raise ValueError('Metadata must be a dictionary.')

    return metadata, document


def find_documents(source: Path, destination: Path) \
        -> Generator[tuple[Path, Path], None, None]:
    """
    Locate all Marquedown documents in the source directory and its subdirectories.
    Yields the source of every document, and the destination for its `index.html`.
    """

    for directory, _, filenames in os.walk(source):
        for fname in filenames:
            file_source = Path(directory) / fname

            if not file_source.is_file():
                continue
            if file_source.suffix not in ('.md', '.mqd'):
                continue

            # File destination from relative path without suffix
            file_relative_path = file_source.relative_to(source)
            file_relative_path = str(file_relative_path)[:-len(file_relative_path.suffix)]
            file_destination = destination / file_relative_path

            # Don't use document name if it is `index.md` or `index.mqd`
            # This should be equivalent to how `index.html` works and be hidden

            if file_source.name in ('index.md', 'index.mqd'):
                yield file_source, file_destination.parent / 'index.html'
            else:
                yield file_source, file_destination / 'index.html'


def render_documents(source: Path, destination: Path, template: Path = None) -> None:
    """
    Load all documents from within the source directory
    and its subdirectories and render them onto a template file.
    Save all renders in the destination directory.
    """

    # Load template
    if template is not None:
        with open(template, 'r') as f:
            template = f.read()

    # Iterate over the source and destination directories for each document
    for fsource, fdestination in find_documents(source, destination):
        # Load document
        with open(fsource, 'r') as f:
            document = f.read()

        # Prepare document and render it
        metadata, document = extract_metadata(document)

        # Ensure destination directory
        os.makedirs(fdestination.parent, exist_ok=True)

        # Create generator for QR code images
        qrgen = QRGenerator(fdestination.parent / 'qr', Path('qr'))

        # Render Marquedown, optionally template, and write to file
        rendered = marquedown(document, qrgen=qrgen)

        with open(fdestination, 'w') as f:
            # Format to template if provided
            if template is not None:
                rendered = template.format_map(defaultdict(str, **metadata, document=rendered))
            # Write to destination
            f.write(rendered)
            print(f'Rendered document: {fsource.relative_to(source)}')




def tagstrip(text: str, tag: str) -> str:
    """
    Strips HTML tag from sides of text.
    """
    
    if contained(text, tag):
        left, right = tags(tag)
        return text.strip()[len(left):-len(right)].strip(" ").strip('\n')

    return text


def contained(text: str, tag: str) -> str:
    """
    Returns whether text is contained within a certain HTML tag.
    """
    left, right = tags(tag)
    stripped_text = text.strip()
    return stripped_text.startswith(left) and stripped_text.endswith(right)


def tags(tag: str) -> str:
    """
    Get left-hand and right-hand sides of a tag.
    """
    return f'<{tag}>', f'</{tag}>'